/**
 *******************************************************************************
 * @file    safety_clock.c
 * @brief   This file provides API functions for Safety Library - CLOCK.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "safety_clock.h"

// use hal.c to determine timeout
#include "txz_hal.h"
   
      
/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup CLOCKTest CLOCK Test
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CLOCKTest_Private_define CLOCKTest Private Define
 *  @{
 */

// OFD BUSY detect timeout (msec)
#define BUSY_TIMEOUT    (10)

// Check period (msec)
#define CHECK_PERIOD    (100)


/** 
 *  @}
 */ /* End of group CLOCKTest_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CLOCKTest_Private_define CLOCKTest Private Define
 *  @{
 */
 
/** ---------------------------------------
 *  @defgroup OFDCR1 OFDCR1
 *  @brief    OFD Control Register 1.
 *  @{
 */

/** 
 *  @defgroup OFDCR1_OFDWEN OFDWEN
 *  @brief    [Bit7:0]: Register Write Enable/Disable Control.
 *  @{
 */
#define REG_OFD_OFDCR1_OFDWEN_DISABLE        ((uint32_t)0x00000006)        /*!< Register Write Disable.     */
#define REG_OFD_OFDCR1_OFDWEN_ENABLE         ((uint32_t)0x000000F9)        /*!< Register Write Enable.      */
/**
 *  @}
 */ /* End of group OFDCR1_OFDWEN */

/**
 *  @}
 */ /* End of group OFDCR1 */
/** ---------------------------------------
 *  @defgroup OFDCR2 OFDCR2
 *  @brief    OFD Control Register 2.
 *  @{
 */

/** 
 *  @defgroup OFDCR2_OFDEN OFDEN
 *  @brief    [Bit7:0]: Frequency Detection Enable/Disable Control.
 *  @{
 */
#define REG_OFD_OFDCR2_OFDEN_DISABLE         ((uint32_t)0x00000000)        /*!< Frequency Detection Disable.  */
#define REG_OFD_OFDCR2_OFDEN_ENABLE          ((uint32_t)0x000000E4)        /*!< Frequency Detection Enable.   */
/**
 *  @}
 */ /* End of group OFDCR2_OFDEN */
/**
 *  @}
 */ /* End of group OFDCR2 */
/** ---------------------------------------
 *  @defgroup OFDRST OFDRST
 *  @brief    OFD RESET Control Register.
 *  @{
 */

/** 
 *  @defgroup OFDRST_OFDRSTEN OFDRSTEN
 *  @brief    [Bit0]: RESET Enable/Disable Control.
 *  @{
 */
#define REG_OFD_OFDRST_OFDRSTEN_DISABLE      ((uint32_t)0x00000000)        /*!< RESET Disable.              */
#define REG_OFD_OFDRST_OFDRSTEN_ENABLE       ((uint32_t)0x00000001)        /*!< RESET Enable.               */
/**
 *  @}
 */ /* End of group OFDCR2_OFD2EN */
 /**
 *  @}
 */ /* End of group OFDRST */

/** ---------------------------------------
 *  @defgroup OFDSTAT OFDSTAT
 *  @brief    OFD Status Register.
 *  @{
 */

/** 
 *  @defgroup OFDSTAT_OFDBUSY OFDBUSY
 *  @brief    [Bit1]: ODD Running status.
 *  @{
 */
#define REG_OFD_OFDSTAT_OFDBUSY_MASK         ((uint32_t)0x00000002)        /*!< For bit MASK.                 */
#define REG_OFD_OFDSTAT_OFDBUSY_SUSPENDED    ((uint32_t)0x00000000)        /*!< OFD Susupended.               */
#define REG_OFD_OFDSTAT_OFDBUSY_RUNNING      ((uint32_t)0x00000002)        /*!< OFD Running.                  */
/**
 *  @}
 */ /* End of group OFDSTAT_OFDBUSY */
/** 
 *  @defgroup OFDSTAT_FRQERR FRQERR
 *  @brief    [Bit0]: ODD Error Detection Flag.
 *  @{
 */
#define REG_OFD_OFDSTAT_FRQERR_MASK          ((uint32_t)0x00000001)        /*!< For bit MASK.                 */
#define REG_OFD_OFDSTAT_FRQERR_NOERROR       ((uint32_t)0x00000000)        /*!< Not Error occurred.           */
#define REG_OFD_OFDSTAT_FRQERR_ERROR         ((uint32_t)0x00000001)        /*!< Error occurred.               */
/**
 *  @}
 */ /* End of group OFDSTAT_OFDBUSY */
 /**
 *  @}
 */ /* End of group OFDSTAT */
/** ---------------------------------------
 *  @defgroup OFDMON1 OFDMON1
 *  @brief    OFD Detection Monitor Clock Setting.
 *  @{
 */

/** 
 *  @defgroup OFDMON1_OFDMON1 OFDMON1
 *  @brief    [Bit0]: Frequency Detection Enable/Disable Control.
 *  @{
 */
#define REG_OFD_OFDMON1_OFDMON1_EHOSC           ((uint32_t)0x00000000)        /*!< Monitor Clock for EHOSC.      */
#define REG_OFD_OFDMON1_OFDMON1_FC              ((uint32_t)0x00000001)        /*!< Monitor Clock for fc.         */
/**
 *  @}
 */ /* End of group OFDMON1_OFDMON1 */
 /**
 *  @}
 */ /* End of group OFDMON1 */


// clock parameter MAX value
#define CLOCK_MAX	(0x0fff)
   
   
   
/**
 *  @}
 */ /* End of group CLOCKTest_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CLOCKTest_Private_define CLOCKTest Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group CLOCKTest_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CLOCKTest_Private_typedef CLOCKTest Private Typedef
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group CLOCKTest_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup CLOCKTest_Private_fuctions CLOCKTest Private Fuctions
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group CLOCKTest_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup ANALOG_IN_Test_Private_typedef ANALOG_IN_Test Private Typedef
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_clock1[41];
        char DEBUGMSG_clock2[50];
        char DEBUGMSG_clock3[49];
        char DEBUGMSG_clock4[42];
        char DEBUGMSG_clock5[43];
        char DEBUGMSG_clock6[31];
        char DEBUGMSG_clock7[65];
        char DEBUGMSG_clock8[50];
        char DEBUGMSG_clock9[54];
    } debugmsg_t;
#endif

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t clock;
    
    char DEBUGMSG_clock_temp[9][65] = {{"safety_CLOCK: illegal clock id: clock_id"}, 
                                     {"safety_CLOCK: max clock out of range (high_clock)"}, 
                                     {"safety_CLOCK: min clock out of range (low_clock)"}, 
                                     {"safety_CLOCK: OFD reset detect. clear it."}, 
                                     {"safety_CLOCK: IHOSC2 not started. start it."},
                                     {"safety_CLOCK: OFDBUSY timeout."}, 
                                     {"safety_CLOCK: BUSY detected in hal_get_tick() - atart_tick msec."},
                                     {"safety_CLOCK: FRQERR detect in CHECK_PERIOD msec."},
                                     {"safety_CLOCK: FRQERR NOT detect in CHECK_PERIOD msec."}};
#endif
/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup CLOCKTest_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     CLOCK CRC test
  * @retval    true  :Success.
  * @note      -
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/
   
bool safety_CLOCK(uint32_t clock_id, uint32_t high_clock, uint32_t low_clock)
{
  uint32_t start_tick;
  
  // OFD controller to use
  TSB_OFD_TypeDef *p_reg_ofd = TSB_OFD;
          
  // RESET controller to use
  TSB_RLM_TypeDef *p_reg_reset = TSB_RLM;

  // Clock Generatror to use
  TSB_CG_TypeDef *p_reg_cg = TSB_CG;
  
  //
  // parameter check
  //
  
  // check clock_id
  // REG_OFD_OFDMON1_OFDMON1_FC = 1
  // REG_OFD_OFDMON1_OFDMON1_EHOSC = 0
  if ((clock_id != REG_OFD_OFDMON1_OFDMON1_FC) && (clock_id != REG_OFD_OFDMON1_OFDMON1_EHOSC)) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("safety_CLOCK: illegal clock id: %d\r\n", clock_id));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(clock.DEBUGMSG_clock1,&DEBUGMSG_clock_temp[0][0]);
#endif
      return false;
  }

  // check high_clock and low_clock : 12 bits
  // we don't check high > low, but just set these values to OFD
  if (high_clock > CLOCK_MAX) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("safety_CLOCK: max clock out of range (%d)\r\n", high_clock));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(clock.DEBUGMSG_clock2,&DEBUGMSG_clock_temp[1][0]);
#endif
      return false;
  }

  if (low_clock > CLOCK_MAX) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("safety_CLOCK: min clock out of range (%d)\r\n", low_clock));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(clock.DEBUGMSG_clock3,&DEBUGMSG_clock_temp[2][0]);
#endif
      return false;
  }

  // clear OFD reset flag (if needed)
  if ((p_reg_reset->RSTFLG1 & 0x08) != 0) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("safety_CLOCK: OFD reset detect. clear it.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(clock.DEBUGMSG_clock4,&DEBUGMSG_clock_temp[3][0]);
#endif
      p_reg_reset->RSTFLG1 &= (uint8_t)(~0x08);	// clear OFD reset
      // continue execution
  }

  // check IHOSC2 for OFD reference clock (10MHz mid precision) 
  if ((p_reg_cg->OSCCR & 0x08) == 0) {
      // start IHOSC2
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("safety_CLOCK: IHOSC2 not started. start it.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(clock.DEBUGMSG_clock5,&DEBUGMSG_clock_temp[4][0]);
#endif
      p_reg_cg->OSCCR |= (uint32_t)0x08;
      // continue execution
  }
    
  //
  // setup OFD
  //
  
  // OFD register write enable (0xf9)
  p_reg_ofd->CR1 = REG_OFD_OFDCR1_OFDWEN_ENABLE;
  
  // OFD frequenct detect disable (0x00)
  p_reg_ofd->CR2 = REG_OFD_OFDCR2_OFDEN_DISABLE;

  // OFD reset output disable
  p_reg_ofd->RST = REG_OFD_OFDRST_OFDRSTEN_DISABLE;
  
  // OFD select monitor clock (fc or EHOSC)
  p_reg_ofd->MON = clock_id;
  
  // OFD set frequency parameter
  if (clock_id == REG_OFD_OFDMON1_OFDMON1_FC) {
      // check fc
      p_reg_ofd->MN1 = low_clock;
      p_reg_ofd->MX1 = high_clock;
  }
  else {
      // check EHOSC
      p_reg_ofd->MN0 = low_clock;
      p_reg_ofd->MX0 = high_clock;
  }        

  // OFD frequenct detect enable (0xe4)
  p_reg_ofd->CR2 = REG_OFD_OFDCR2_OFDEN_ENABLE;
  	  	
  // OFD register write disable (0x06)
  p_reg_ofd->CR1 = REG_OFD_OFDCR1_OFDWEN_DISABLE;
  
  //
  // wait for OFD BUSY set
  //
  start_tick = hal_get_tick();
  while (((p_reg_ofd->STAT & REG_OFD_OFDSTAT_OFDBUSY_MASK) != REG_OFD_OFDSTAT_OFDBUSY_RUNNING) && ((hal_get_tick() - start_tick) < BUSY_TIMEOUT)) {
      // wait some time
      __NOP();
  }

  if ((p_reg_ofd->STAT & REG_OFD_OFDSTAT_OFDBUSY_MASK) != REG_OFD_OFDSTAT_OFDBUSY_RUNNING) {
      // timeout detect
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("safety_CLOCK: OFDBUSY timeout.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(clock.DEBUGMSG_clock6,&DEBUGMSG_clock_temp[5][0]);
#endif
      return false;
  }
  
  // for DEBUG
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("safety_CLOCK: BUSY detected in %d msec.\r\n", hal_get_tick() - start_tick));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
  strcpy(clock.DEBUGMSG_clock7,&DEBUGMSG_clock_temp[6][0]);
#endif


  //
  // check OFD error for a while
  //
  start_tick = hal_get_tick();
  while (((p_reg_ofd->STAT & REG_OFD_OFDSTAT_FRQERR_MASK) != REG_OFD_OFDSTAT_FRQERR_ERROR) &&  ((hal_get_tick() - start_tick) < CHECK_PERIOD)) {
      // wait some time
      __NOP();
  }

  if ((p_reg_ofd->STAT & REG_OFD_OFDSTAT_FRQERR_MASK) != REG_OFD_OFDSTAT_FRQERR_NOERROR) {
      // error detect
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("safety_CLOCK: FRQERR detect in %d msec.\r\n", CHECK_PERIOD));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(clock.DEBUGMSG_clock8,&DEBUGMSG_clock_temp[7][0]);
#endif
      return false;
  }

  // for DEBUG
#if defined ( __ICCARM__ )    /*IAR Compiler */
  DEBUGMSG(1, ("safety_CLOCK: FRQERR NOT detect in %d msec.\r\n", CHECK_PERIOD));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(clock.DEBUGMSG_clock9,&DEBUGMSG_clock_temp[8][0]);
#endif
  
  return true;		// CLOCK out of range not detect
}

/**
 *  @}
 */ /* End of group CLOCKTest_Exported_functions */

/**
 *  @} 
 */ /* End of group CLOCKTest */

/**
 *  @} 
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */

