/**
 *******************************************************************************
 * @file    safety_cpu_reg.c
 * @brief   This file provides API functions for Safety Library - CPU Register.
 * @version V0.1.0.2
 * $Date:: 2019-02-014 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include "safety_cpu_reg.h"

#if defined ( __CC_ARM )        /* RealView Compiler */

/* note: these values must be the value used as "flexible second opearnd (Operand2) - constant" */
/* see http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.dui0802bj/CIHEAFJC.html */

__ASM bool safety_CPU_Register(uint32_t *result_word)
{
    /* bool safety_CPU_Register(uint32_t *p_result_dword) */
    /* r0 = ptr to result_word */
    /* lr = return address */

    /* backup registers */
    /* still we can use r1 - r3 */
    push    {r0, r4-r12, lr}

    /* test r0 */
    /* special case: we only use r0 for this test */
    mov r0, #0x55555555
    cmp r0, #0x55555555
    bne test_r0_error
    mov r0, #0xaaaaaaaa
    cmp r0, #0xaaaaaaaa
    bne test_r0_error
	
    /* test r0 success */
    b   test_r1
}

__ASM void test_r0_error(void)
{
    mov r0, #1        /* bit0 = 1 */
    b    error_detect0
}

__ASM void test_r1(void)
{
    /* test r1 */
    mov r0, #2
    mov r1, #0x55555555
    cmp r1, #0x55555555
    bne error_detect0
    mov r1, #0xaaaaaaaa
    cmp r1, #0xaaaaaaaa
    bne	error_detect0

    /* test r2 */
    mov r0, #4
    mov r2, #0x55555555
    cmp r2, #0x55555555
    bne error_detect0
    mov r2, #0xaaaaaaaa
    cmp r2, #0xaaaaaaaa
    bne	error_detect0

    /* test r3 */
    mov r0, #8
    mov r3, #0x55555555
    cmp r3, #0x55555555
    bne error_detect0
    mov r3, #0xaaaaaaaa
    cmp r3, #0xaaaaaaaa
    bne error_detect0

    /* test r4 */
    mov r0, #0x10
    mov r4, #0x55555555
    cmp r4, #0x55555555
    bne error_detect0
    mov r4, #0xaaaaaaaa
    cmp r4, #0xaaaaaaaa
    bne	error_detect0

    /* test r5 */
    mov r0, #0x20
    mov r5, #0x55555555
    cmp r5, #0x55555555
    bne error_detect0
    mov r5, #0xaaaaaaaa
    cmp r5, #0xaaaaaaaa
    bne error_detect0

    /* test r6 */
    mov r0, #0x40
    mov r6, #0x55555555
    cmp r6, #0x55555555
    bne error_detect0
    mov r6, #0xaaaaaaaa
    cmp r6, #0xaaaaaaaa
    bne error_detect0

    /* test r7 */
    mov r0, #0x80
    mov r7, #0x55555555
    cmp r7, #0x55555555
    bne error_detect0
    mov r7, #0xaaaaaaaa
    cmp r7, #0xaaaaaaaa
    bne error_detect0

    /* test r8 */
    mov r0, #0x100
    mov r8, #0x55555555
    cmp r8, #0x55555555
    bne error_detect0
    mov r8, #0xaaaaaaaa
    cmp r8, #0xaaaaaaaa
    bne error_detect0
        
    /* test r9 */
    mov r0, #0x200
    mov r9, #0x55555555
    cmp r9, #0x55555555
    bne error_detect0
    mov r9, #0xaaaaaaaa
    cmp r9, #0xaaaaaaaa
    bne error_detect0

    /* test r10 */
    mov r0, #0x400
    mov r10, #0x55555555
    cmp r10, #0x55555555
    bne error_detect0
    mov r10, #0xaaaaaaaa
    cmp r10, #0xaaaaaaaa
        
    /* bne    error_detect0 */
    beq     test_r11        /* SUCCESS. go to next */
}

/* this is here to prevent from  near jump out of range */
__ASM void error_detect0(void)
{
    b       error_detect1
}

__ASM void test_r11(void)
{
    /* test r11 */
    mov r0, #0x800
    mov r11, #0x55555555
    cmp r11, #0x55555555
    bne error_detect1
    mov r11, #0xaaaaaaaa
    cmp r11, #0xaaaaaaaa
    bne error_detect1

    /* test r12 */
    mov r0, #0x1000
    mov r12, #0x55555555
    cmp r12, #0x55555555
    bne error_detect1
    mov r12, #0xaaaaaaaa
    cmp r12, #0xaaaaaaaa
    bne error_detect1

    /* test Main SP */
    /* assume this is not user mode */
    /* note: IRQ must be disabled */
    mov r0, #0x2000
    mov r2, sp                  /* backup main sp */

    mov r1, #0x55555555
    and r1, r1, #0xfffffffc       /* set lower 2 bits to 0 */
    mov sp, r1
    cmp sp, r1
    bne test_main_sp_error

    mov r1, #0xaaaaaaaa
    and r1, r1, #0xfffffffc       /* set lower 2 bits to 0 */
    mov sp, r1
    cmp sp, r1
    bne	test_main_sp_error

    /* test MAIN sp success */
    mov sp, r2                  /* restore main sp */
    b   test_proc_sp
}

__ASM void test_main_sp_error(void)
{
    mov sp, r2                  /* restore main sp */
    b   error_detect1
}

__ASM void test_proc_sp(void)
{
    /* test Process SP */
    /* note: task switch must be disabled. use IRQ disable for this purpose. */
    mov r0, #0x4000
    mrs r2, psp                 /* backup process sp */

    mov r1, #0x55555555
    and r1, r1, #0xfffffffc       /* set lower 2 bits to 0 */
    msr psp, r1
    mrs r3, psp
    cmp r3, r1
    bne test_proc_sp_error

    mov r1, #0xaaaaaaaa
    and r1, r1, #0xfffffffc       /* set lower 2 bits to 0 */
    msr psp, r1
    mrs r3, psp
    cmp r3, r1
	bne	test_proc_sp_error

    /* test PROC sp success */
    msr psp, r2                 /* restore process sp */
    b   test_lr
}

__ASM void test_proc_sp_error(void)
{
    msr psp, r2                 /* restore process sp */
    b   error_detect1
}

__ASM void test_lr(void)
{
    /* test lr */
    mov r0, #0x8000
    mov r5, lr
    mov lr, #0x55555555
    cmp lr, #0x55555555
    bne error_detect1
    mov lr, #0xaaaaaaaa
    cmp lr, #0xaaaaaaaa
    bne error_detect1

    /* test CPSR(upper 5 bits) */
    /* use "APSR" for access upper 5 bits of CPSR */
    /* APSR no need to back up */
    mov r0, #0x10000

    mov r1, #0x55555555
    and r1, r1, #0xf8000000      /* mask unused bits */
    msr apsr, r1
    mrs r3, apsr
    cmp r3, r1
    bne error_detect1

    mov r1, #0xaaaaaaaa
    and r1, r1, #0xf8000000     /* mask unused bits */
    msr apsr, r1
    mrs r3, apsr
    cmp r3, r1
    bne	error_detect1

    /* no error detect */
    mov r0, #1
}

__ASM void error_detect1(void)
{
    /* restore registers */
    /* r1 = p_result_word */
    pop {r1, r4-r12, lr}

    /* store error bitmask */
    cmp r1, #0
    beq error_detect2
    str r0, [r1]
}

__ASM void error_detect2(void)
{
    /* return result (if r1 == 0 true) */
    mov r1, r0
    mov r0, #1
    cmp r1, #0
    b error_detect3

    mov r0, #0
}

__ASM void error_detect3(void)
{
    uxtb r0, r0	    /* zero extend */
    bx lr
}
#elif defined ( __ICCARM__ )    /* IAR Compiler */

/* note: these values must be the value used as "flexible second opearnd (Operand2) - constant" */
/* see http://infocenter.arm.com/help/index.jsp?topic=/com.arm.doc.dui0802bj/CIHEAFJC.html */

bool safety_CPU_Register(uint32_t *result_word)
{
    uint8_t ret;
    
    /* bool safety_CPU_Register(uint32_t *p_result_dword) */
    /* r0 = ptr to result_word */
    /* lr = return address */

    /* backup registers */
    /* still we can use r1 - r3 */
    asm("push    {r0, r4-r12, lr}");

    /* test r0 */
    /* special case: we only use r0 for this test */
    
    asm("mov r0, #0x55555555");
    asm("cmp r0, #0x55555555");
    asm("bne test_r0_error");
    asm("mov r0, #0xaaaaaaaa");
    asm("cmp r0, #0xaaaaaaaa");
    asm("bne test_r0_error");
	
    /* test r0 success */
    asm("b   test_r1");
    
    asm("mov %0, lr" : "=r"(ret));
    
    return ret;
}

void test_r0_error(void)
{
    asm("mov r0, #1");        /* bit0 = 1 */
    asm("b    error_detect0");
}

void test_r1(void)
{
    /* test r1 */
    asm("mov r0, #2");
    asm("mov r1, #0x55555555");
    asm("cmp r1, #0x55555555");
    asm("bne error_detect0");
    asm("mov r1, #0xaaaaaaaa");
    asm("cmp r1, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r2 */
    asm("mov r0, #4");
    asm("mov r2, #0x55555555");
    asm("cmp r2, #0x55555555");
    asm("bne error_detect0");
    asm("mov r2, #0xaaaaaaaa");
    asm("cmp r2, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r3 */
    asm("mov r0, #8");
    asm("mov r3, #0x55555555");
    asm("cmp r3, #0x55555555");
    asm("bne error_detect0");
    asm("mov r3, #0xaaaaaaaa");
    asm("cmp r3, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r4 */
    asm("mov r0, #0x10");
    asm("mov r4, #0x55555555");
    asm("cmp r4, #0x55555555");
    asm("bne error_detect0");
    asm("mov r4, #0xaaaaaaaa");
    asm("cmp r4, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r5 */
    asm("mov r0, #0x20");
    asm("mov r5, #0x55555555");
    asm("cmp r5, #0x55555555");
    asm("bne error_detect0");
    asm("mov r5, #0xaaaaaaaa");
    asm("cmp r5, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r6 */
    asm("mov r0, #0x40");
    asm("mov r6, #0x55555555");
    asm("cmp r6, #0x55555555");
    asm("bne error_detect0");
    asm("mov r6, #0xaaaaaaaa");
    asm("cmp r6, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r7 */
    asm("mov r0, #0x80");
    asm("mov r7, #0x55555555");
    asm("cmp r7, #0x55555555");
    asm("bne error_detect0");
    asm("mov r7, #0xaaaaaaaa");
    asm("cmp r7, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r8 */
    asm("mov r0, #0x100");
    asm("mov r8, #0x55555555");
    asm("cmp r8, #0x55555555");
    asm("bne error_detect0");
    asm("mov r8, #0xaaaaaaaa");
    asm("cmp r8, #0xaaaaaaaa");
    asm("bne error_detect0");
        
    /* test r9 */
    asm("mov r0, #0x200");
    asm("mov r9, #0x55555555");
    asm("cmp r9, #0x55555555");
    asm("bne error_detect0");
    asm("mov r9, #0xaaaaaaaa");
    asm("cmp r9, #0xaaaaaaaa");
    asm("bne error_detect0");

    /* test r10 */
    asm("mov r0, #0x400");
    asm("mov r10, #0x55555555");
    asm("cmp r10, #0x55555555");
    asm("bne error_detect0");
    asm("mov r10, #0xaaaaaaaa");
    asm("cmp r10, #0xaaaaaaaa");
        
    /* bne    error_detect0 */
    asm("beq     test_r11");        /* SUCCESS. go to next */
}

/* this is here to prevent from  near jump out of range */
void error_detect0(void)
{
    asm("b       error_detect1");
}

void test_r11(void)
{
    /* test r11 */
    asm("mov r0, #0x800");
    asm("mov r11, #0x55555555");
    asm("cmp r11, #0x55555555");
    asm("bne error_detect1");
    asm("mov r11, #0xaaaaaaaa");
    asm("cmp r11, #0xaaaaaaaa");
    asm("bne error_detect1");

    /* test r12 */
    asm("mov r0, #0x1000");
    asm("mov r12, #0x55555555");
    asm("cmp r12, #0x55555555");
    asm("bne error_detect1");
    asm("mov r12, #0xaaaaaaaa");
    asm("cmp r12, #0xaaaaaaaa");
    asm("bne error_detect1");

    /* test Main SP */
    /* assume this is not user mode */
    /* note: IRQ must be disabled */
    asm("mov r0, #0x2000");
    asm("mov r2, sp");                  /* backup main sp */

    asm("mov r1, #0x55555555");
    asm("and r1, r1, #0xfffffffc");       /* set lower 2 bits to 0 */
    asm("mov sp, r1");
    asm("cmp sp, r1");
    asm("bne test_main_sp_error");

    asm("mov r1, #0xaaaaaaaa");
    asm("and r1, r1, #0xfffffffc");       /* set lower 2 bits to 0 */
    asm("mov sp, r1");
    asm("cmp sp, r1");
    asm("bne	test_main_sp_error");

    /* test MAIN sp success */
    asm("mov sp, r2");                  /* restore main sp */
    asm("b   test_proc_sp");
}

void test_main_sp_error(void)
{
    asm("mov sp, r2");                  /* restore main sp */
    asm("b   error_detect1");
}

void test_proc_sp(void)
{
    /* test Process SP */
    /* note: task switch must be disabled. use IRQ disable for this purpose. */
    asm("mov r0, #0x4000");
    asm("mrs r2, psp");                 /* backup process sp */

    asm("mov r1, #0x55555555");
    asm("and r1, r1, #0xfffffffc");       /* set lower 2 bits to 0 */
    asm("msr psp, r1");
    asm("mrs r3, psp");
    asm("cmp r3, r1");
    asm("bne test_proc_sp_error");

    asm("mov r1, #0xaaaaaaaa");
    asm("and r1, r1, #0xfffffffc");       /* set lower 2 bits to 0 */
    asm("msr psp, r1");
    asm("mrs r3, psp");
    asm("cmp r3, r1");
    asm("bne    test_proc_sp_error");

    /* test PROC sp success */
    asm("msr psp, r2");                 /* restore process sp */
    asm("b   test_lr");
}

void test_proc_sp_error(void)
{
    asm("msr psp, r2");                 /* restore process sp */
    asm("b   error_detect1");
}

void test_lr(void)
{
    /* test lr */
    asm("mov r0, #0x8000");
    asm("mov r5, lr");
    asm("mov lr, #0x55555555");
    asm("cmp lr, #0x55555555");
    asm("bne error_detect1");
    asm("mov lr, #0xaaaaaaaa");
    asm("cmp lr, #0xaaaaaaaa");
    asm("bne error_detect1");

    /* test CPSR(upper 5 bits) */
    /* use "APSR" for access upper 5 bits of CPSR */
    /* APSR no need to back up */
    asm("mov r0, #0x10000");

    asm("mov r1, #0x55555555");
    asm("and r1, r1, #0xf8000000");      /* mask unused bits */
    asm("msr apsr, r1");
    asm("mrs r3, apsr");
    asm("cmp r3, r1");
    asm("bne error_detect1");

    asm("mov r1, #0xaaaaaaaa");
    asm("and r1, r1, #0xf8000000");     /* mask unused bits */
    asm("msr apsr, r1");
    asm("mrs r3, apsr");
    asm("cmp r3, r1");
    asm("bne	error_detect1");


    /* no error detect */
    asm("mov r0, #1");
    asm("mov r4, #0");
    asm("mov lr, r5");
}

void error_detect1(void)
{
    /* restore registers */
    /* r1 = p_result_word */
    asm("pop {r1, r4-r12, lr}");

    /* store error bitmask */
    asm("cmp r1, #0");
    asm("beq error_detect2");
    asm("str r0, [r1]");
}

void error_detect2(void)
{
    /* return result (if r1 == 0 true) */
    asm("mov r1, r0");
    asm("mov r0, #1");
    asm("cmp r1, #0");
    asm("b error_detect3");

    asm("mov r0, #0");
}

void error_detect3(void)
{
    asm("uxtb r0, r0");	    /* zero extend */
    asm("bx lr");
}
#endif

#ifdef __cplusplus
}
#endif /* __cplusplus */
