/**
 *******************************************************************************
 * @file    safety_flash.c
 * @brief   This file provides API functions for Safety Library - FLASH.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include "safety_flash.h"

/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup FLASHTest FLASH Test
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASHTest_Private_define FLASHTest Private Define
 *  @{
 */

/* no define */

/** 
 *  @}
 */ /* End of group FLASHTest_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASHTest_Private_define FLASHTest Private Define
 *  @{
 */
 
/** 
 *  @defgroup CRCTYP_CFMT CFMT
 *  @brief    [Bit3:2]: CRC Format Select.
 *  @{
 */
#define REG_CRC_CFMT_CRC16                  ((uint32_t)0x00000000)        /*!< CRC16 Format.                                */
#define REG_CRC_CFMT_CRC32                  ((uint32_t)0x0000000C)        /*!< CRC32 Fromat.                                */

/**
 *  @}
 */ /* End of group CRCTYP_CFMT */
/** 
 *  @defgroup CRCTYP_DBIT DBIT
 *  @brief    [Bit1:0]: CRC Format Select.
 *  @{
 */
#define REG_CRC_DBIT_32bit                  ((uint32_t)0x00000000)        /*!< Input data 32bit.                            */
#define REG_CRC_DBIT_8bit                   ((uint32_t)0x00000001)        /*!< Input data 8bit.                             */
#define REG_CRC_DBIT_16bit                  ((uint32_t)0x00000002)        /*!< Input data 16bit.                            */

/**
 *  @}
 */ /* End of group CRCTYP_DBIT */

/** 
 *  @defgroup CRCCLC_CRCCLC CRCCLC
 *  @brief    [Bit31:0]: CRC Calculation Result Store.
 *  @{
 */
#define REG_CRCCLC_DEFAULT_VAL_16           ((uint32_t)0x0000FFFF)        /*!< Default Value for 16bit.                     */
#define REG_CRCCLC_DEFAULT_VAL_32           ((uint32_t)0xFFFFFFFF)        /*!< Default Value for 32bit.                     */

/**
 *  @}
 */ /* End of group CRCCLC_CRCCLC */

/**
 *  @}
 */ /* End of group FLASHTest_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASHTest_Private_define FLASHTest Private Define
 *  @{
 */
 
/* no define */
 
/**
 *  @}
 */ /* End of group FLASHTest_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASHTest_Private_typedef FLASHTest Private Typedef
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_flash1[42];
        char DEBUGMSG_flash2[42];
        char DEBUGMSG_flash3[52];
    } debugmsg_t;
#endif

/**
 *  @}
 */ /* End of group FLASHTest_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_variables DIGITAL IO Test Private Variables
 *  @{
 */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t flash;
    
    char DEBUGMSG_flash_temp[3][52] = {{"get_CRC32: start=0xstart, length=0xlength"}, 
                                       {"get_CRC16: start=0xstart, length=0xlength"}, 
                                       {"safety_FLASH: value=0xcrc32, provided=0xcrc32_value"}};
#endif
/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup FLASHTest_Private_fuctions FLASHTest Private Fuctions
 *  @{
 */

#if CRC == CRC32
static uint32_t get_CRC32(uint32_t start, uint32_t length);

// get CRC32
#if defined ( __ICCARM__ )    /* IAR Compiler */
    #pragma optimize=no_inline
#endif
static uint32_t get_CRC32(uint32_t start, uint32_t length)
{
    uint8_t *p;
    
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("get_CRC32: start=0x%08x, length=0x%08x\r\n", start, length));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(flash.DEBUGMSG_flash1,&DEBUGMSG_flash_temp[0][0]);
#endif

    // CRC instance to use
    TSB_CRC_TypeDef *p_reg = TSB_CRC;

    // set CRC parameters
    p_reg->TYP = (REG_CRC_CFMT_CRC32 |			// select CRC32
                  REG_CRC_DBIT_8bit);			// input is 8bit based

    p_reg->CLC = REG_CRCCLC_DEFAULT_VAL_32;		// reset to 32bit default (0xffffffff)

    // loop input
    for (p = (uint8_t *)start; length > 0; p++, length--) {
        p_reg->DIN = (uint32_t)(*p & 0xff);
    }

    // return 32bit CRC
    return p_reg->CLC;	
}
#endif


#if CRC == CRC16
static uint32_t get_CRC16(uint32_t start, uint32_t length);

// get CRC16 in lower 16bits
#if defined ( __ICCARM__ )    /* IAR Compiler */
    #pragma optimize=no_inline
#endif
static uint32_t get_CRC16(uint32_t start, uint32_t length)
{
    uint8_t *p;

#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("get_CRC16: start=0x%08x, length=0x%08x\r\n", start, length));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(flash.DEBUGMSG_flash2,&DEBUGMSG_flash_temp[1][0]);
#endif

    // CRC instance to use
    TSB_CRC_TypeDef *p_reg = TSB_CRC;

    // set CRC parameters
    p_reg->TYP = (REG_CRC_CFMT_CRC16 |			// select CRC16
                  REG_CRC_DBIT_8bit);			// input is 8bit based

    p_reg->CLC = REG_CRCCLC_DEFAULT_VAL_16;		// reset to 16bit default (0x0000ffff)

    // loop input
    for (p = (uint8_t *)start; length > 0; p++, length--) {
        p_reg->DIN = (uint32_t)(*p & 0xff);
    }

    // return 16bit CRC
    return (p_reg->CLC & 0x0000ffff);	
}
#endif

/**
 *  @}
 */ /* End of group FLASHTest_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup FLASHTest_Exported_functions
 *  @{
 */
/*--------------------------------------------------*/
/**
  * @brief     FLASH CRC test
  * @retval    true  :Success.
  * @note      -
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/
bool safety_FLASH(uint32_t start, uint32_t length, uint32_t crc32_value)
{
#if CRC == CRC32
    uint32_t crc32;
    
    //
    // caution : no argument check for "start" and "length" here!!!
    //
    crc32 = get_CRC32(start, length);
    
  #if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("safety_FLASH: value=0x%08x, provided=0x%08x\r\n", 
           crc32, crc32_value));
  #elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(flash.DEBUGMSG_flash3,&DEBUGMSG_flash_temp[2][0]);
  #endif
    
    return crc32 == crc32_value;	// true == OK 
#endif

#if CRC == CRC16
    uint32_t crc16;
    
    //
    // caution : no argument check for "start" and "length" here!!!
    //
    crc16 = get_CRC16(start, length);
    
  #if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("safety_FLASH: value=0x%08x, provided=0x%08x\r\n", 
           crc16, (uint16_t)crc32_value));
  #elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(flash.DEBUGMSG_flash3,&DEBUGMSG_flash_temp[2][0]);
  #endif
  
    return crc16 == (uint16_t)crc32_value;	// true == OK 
#endif
}

/**
 *  @}
 */ /* End of group FLASHTest_Exported_functions */

/**
 *  @} 
 */ /* End of group FLASHTest */

/**
 *  @} 
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */

