/**
 *******************************************************************************
 * @file    safety_interrupt.c
 * @brief   This file provides API functions for Safety Library - Interrupt.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "safety_interrupt.h"

// timer driver definitions
#include "txz_t32a.h"
      
// use timer functions
#include "bsp_timer.h"  

   
/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup INTERRUPT_Test INTERRUPT Test
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup INTERRUPT_Test_Private_define INTERRUPT_Test Private Define
 *  @{
 */

/** 
 *  @}
 */ /* End of group INTERRUPT_Test_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup INTERRUPT_Test_Private_define INTERRUPT_Test Private Define
 *  @{
 */
    
  
/**
 *  @}
 */ /* End of group INTERRUPT_Test_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup INTERRUPT_Test_Private_define INTERRUPT_Test Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group INTERRUPT_Test_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup INTERRUPT_Test_Private_typedef INTERRUPT_Test Private Typedef
 *  @{
 */
 
typedef struct
{
    t32a_t      t32a;
    timer_t     timer;
    uint32_t	irq_count;	// note: watch out for overflow
} information_t;

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_interrupt1[44];
        char DEBUGMSG_interrupt2[54];
        char DEBUGMSG_interrupt3[37];
        char DEBUGMSG_interrupt4[33];
        char DEBUGMSG_interrupt5[65];
        char DEBUGMSG_interrupt6[63];
        char DEBUGMSG_interrupt7[56];
    } debugmsg_t;
#endif

/**
 *  @}
 */ /* End of group INTERRUPT_Test_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup INTERRUPT_Test_Private_variables INTERRUPT_Test Private Variables
 *  @{
 */
static information_t local_instance;
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t interrupt;
    
    char DEBUGMSG_interrupt_temp[7][65] = {{"safety_Interrupt: invalid timer_id timer_id"}, 
                                           {"safety_Interrupt: t32a_mode_init(ch=timer_id) failed."}, 
                                           {"took GetTickCount() - old_tick msec."}, 
                                           {"safety_Interrupt: NO IRQ DETECT."}, 
                                           {"safety_Interrupt: LOW IRQ DETECT. (irq=local_instance.irq_count)"},
                                           {"safety_Interrupt: TOO MANY IRQ. (irq=local_instance.irq_count)"}, 
                                           {"safety_Interrupt: SUCCESS. irq=local_instance.irq_count"}};
#endif
/**
 *  @}
 */ /* End of group INTERRUPT_Test_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup INTERRUPT_Test_Private_fuctions INTERRUPT_Test Private Fuctions
 *  @{
 */
static void timer_interval_handler(uint32_t id);

//
// private IRQ handler for safety_Interrupt
//
static void timer_interval_handler(uint32_t id)
{
    if (id == (uint32_t)&local_instance.timer) {
        // increment IRQ counter
        // watch out for overflow
        if (local_instance.irq_count < 0xffffffff)
        {
            local_instance.irq_count++;
        }
    }
}


/**
 *  @}
 */ /* End of group INTERRUPT_Test_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup INTERRUPT_Test_Exported_functions
 *  @{
 */

//
// external IRQ handler for safety_Interrupt
//
void safety_interrupt_irq_handler(void)
{
    // forward to timer.c handler using local instance
    timer_irq_handler(&local_instance.timer);
}

/*--------------------------------------------------*/
/**
  * @brief     start timer and count how many IRQs occur
  * @retval    0 :failed, 1 :success
  * @note      IRQ handler needed to be set
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/

// 100 usec per IRQ and test for 100msec = 1000 times
#define MIN_THRESHOLD		(500)
#define MAX_THRESHOLD		(2000)

#ifdef OPTIMIZE_HI
#define INTR_LOOP_COUNT      (1316000)
#else
#define INTR_LOOP_COUNT      (707000)
#endif

bool safety_Interrupt(int timer_id, uint32_t *p_result_word)
{
    // T32A driver / timer instance to use
    t32a_t *p_t32a = &local_instance.t32a;
    timer_t *p_timer = &local_instance.timer;
    
    // IRQ level used for timer
    IRQn_Type	timer_irq;
	
    // result bitmask
    uint32_t result_word = 0;

    
    // clear result word for safety
    if (p_result_word != SAFETY_NULL)
    {
        *p_result_word = 0;
    }

    // assign T32A instance by timer_id
    switch (timer_id) {
    case 0:
    	// note: this is used for 1 MS timer by BSP
        p_t32a->p_instance = TSB_T32A0;
        timer_irq = INTT32A0AC_IRQn;
        break;
    case 1:
        p_t32a->p_instance = TSB_T32A1;
        timer_irq = INTT32A1AC_IRQn;
        break;
    case 2:
    	// note: this is used for 1 SEC timer by BSP
        p_t32a->p_instance = TSB_T32A2;
        timer_irq = INTT32A2AC_IRQn;
        break;
    case 3:
        p_t32a->p_instance = TSB_T32A3;
        timer_irq = INTT32A3AC_IRQn;
        break;
    case 4:
        p_t32a->p_instance = TSB_T32A4;
        timer_irq = INTT32A4AC_IRQn;
        break;
    case 5:
        p_t32a->p_instance = TSB_T32A5;
        timer_irq = INTT32A5AC_IRQn;
        break;
    default:
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_Interrupt: invalid timer_id %d.\r\n", timer_id));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(interrupt.DEBUGMSG_interrupt1,&DEBUGMSG_interrupt_temp[0][0]);
#endif
    	return false;
    }
    
    /*--- Initialize T32A ---*/
    p_t32a->init_mode.mode.halt = T32A_DBG_HALT_STOP;
    p_t32a->init_mode.mode.mode = T32A_MODE_16;
    if (t32a_mode_init(p_t32a) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_Interrupt: t32a_mode_init(ch=%d) failed.\r\n", timer_id));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(interrupt.DEBUGMSG_interrupt2,&DEBUGMSG_interrupt_temp[1][0]);
#endif
    	return false;
    }

    /*--- Initialize timer ---*/
    p_timer->init.id       = (uint32_t)&local_instance.timer;
    p_timer->init.p_t32a   = p_t32a;
    p_timer->init.type     = T32A_TIMERA;	// 16bit Timer A register
    p_timer->init.irq      = timer_irq;
    p_timer->init.interval = 100;               // usec per IRQ, 16bit value
    p_timer->init.handler  = timer_interval_handler;

    timer_initialize(p_timer);

    // clear IRQ count
    local_instance.irq_count = 0;
    
    // Start timer    
    timer_start(p_timer);

    // wait for 100msec time here
    // IRQs may occur 1000 times
    // SysTick IRQ used for LOOP_COUNT caliblation
    {
#ifdef TEST
      uint32_t old_tick = GetTickCount();
#endif
      
      volatile uint32_t status;      
      int i;
      
      for (i = 0; i < INTR_LOOP_COUNT; i++) {
        // read TIMERA: status register
        status = p_t32a->p_instance->STA;
      }
      
#ifdef TEST
      
    #if defined ( __ICCARM__ )    /*IAR Compiler */
          DEBUGMSG(1, ("took %d msec.\r\n", GetTickCount() - old_tick));
    #elif defined ( __CC_ARM   )      /* RealView Compiler */
          strcpy(interrupt.DEBUGMSG_interrupt3,&DEBUGMSG_interrupt_temp[2][0]);
    #endif

#endif
    }

    // Stop timer
    timer_stop(p_timer);

    // finalize timer
    timer_finalize(p_timer);

    // determine how many times IRQ handled
    if (local_instance.irq_count == 0) {
        // no IRQ detect
        result_word = (1 << 0);	// bit0 set
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_Interrupt: NO IRQ DETECT.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(interrupt.DEBUGMSG_interrupt4,&DEBUGMSG_interrupt_temp[3][0]);
#endif
    }
    else if (local_instance.irq_count < MIN_THRESHOLD) {
        // low IRQ times
        result_word = (1 << 1);	// bit1 set
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_Interrupt: LOW IRQ DETECT. (irq=%d)\r\n", local_instance.irq_count));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(interrupt.DEBUGMSG_interrupt5,&DEBUGMSG_interrupt_temp[4][0]);
#endif
    }
    else if (MAX_THRESHOLD < local_instance.irq_count) {
        // too many IRQ times
        result_word = (1 << 2);	// bit2 set
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_Interrupt: TOO MANY IRQ. (irq=%d)\r\n", local_instance.irq_count));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(interrupt.DEBUGMSG_interrupt6,&DEBUGMSG_interrupt_temp[5][0]);
#endif
    }
    else {
        // OK
        // result_word == 0
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_Interrupt: SUCCESS. irq=%d\r\n", local_instance.irq_count));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(interrupt.DEBUGMSG_interrupt7,&DEBUGMSG_interrupt_temp[6][0]);
#endif
    }

    // return result
    if (p_result_word != SAFETY_NULL)
    {
        *p_result_word = result_word;
    }

    return (result_word == 0) ? true : false;	
}

/**
 *  @}
 */ /* End of group INTERRUPT_Test_Exported_functions */

/**
 *  @} 
 */ /* End of group INTERRUPT_Test */

/**
 *  @} 
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */

