/**
 *******************************************************************************
 * @file    safety_uart.c
 * @brief   This file provides API functions for Safety Library - UART.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

#ifdef __cplusplus
 extern "C" {
#endif

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "safety_uart.h"
   
// use uart_io header for internal typedefs
#include "bsp_uart_io.h"

// use hal.h to determine timeout
#include "txz_hal.h"
      
/**
 *  @addtogroup Safety_Library Safety Library
 *  @{
 */

/** 
 *  @addtogroup UARTTest UART Test
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Private_define UARTTest Private Define
 *  @{
 */

static uart_io_t local_instance;  /*!< USB UART Instance.  */

/** 
 *  @}
 */ /* End of group UARTTest_Private_define */

/*------------------------------------------------------------------------------*/
/*  Macro Definition                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Private_define UARTTest Private Define
 *  @{
 */
 
// bps value used to test
#define SAFETY_TEST_BPS           115200

// use UART internal loopback? (CR0.LPB)
#define USE_INTERNAL_LOOPBACK   1


/**
 *  @}
 */ /* End of group UARTTest_Private_define */


/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Private_define UARTTest Private Define
 *  @{
 */

/* no define */

/**
 *  @}
 */ /* End of group UARTTest_Private_define */

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Private_typedef UARTTest Private Typedef
 *  @{
 */

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_uart1[30];
        char DEBUGMSG_uart2[34];
        char DEBUGMSG_uart3[18];
        char DEBUGMSG_uart4[23];
        char DEBUGMSG_uart5[29];
        char DEBUGMSG_uart6[56];
        char DEBUGMSG_uart7[41];
        char DEBUGMSG_uart8[57];
        char DEBUGMSG_uart9[78];
        char DEBUGMSG_uart10[31];
        char DEBUGMSG_uart11[56];
    } debugmsg_t;
#endif

/**
 *  @}
 */ /* End of group UARTTest_Private_typedef */

/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup DIGITAL_IO_Test_Private_variables DIGITAL IO Test Private Variables
 *  @{
 */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t uart;
    
    char DEBUGMSG_uart_temp[11][78] = {{"invalid UART ID (p_param->id)"}, 
                                      {"uart_get_boudrate_setting FAILED."},
                                      {"uart_init FAILED."},
                                      {"uart_receiveId FAILED."},
                                      {"safety_UART_finalize FAILED."},
                                      {"safety_UART_loopback: timeout set to timeout_tick msec."},
                                      {"safety_UART_loopback: initialize FAILED."},
                                      {"safety_UART_loopback: TIMEOUT after timeout_tick msec."},
                                      {"p_end - p_send bytes sent, p_end - p_recv bytes received. length bytes total."},
                                      {"safety_UART_loopback: SUCCESS."},
                                      {"timeout = timeout_tick msec, actual = actual_tick msec."}};
#endif
/**
 *  @}
 */ /* End of group DIGITAL_IO_Test_Private_variables */

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
/** 
 *  @defgroup UARTTest_Private_fuctions UARTTest Private Fuctions
 *  @{
 */

// functions from "uart_io.c"
static void safety_UART_error(void);
static int put_char(int c);
static int get_char(void);
static void transmit_handler(uint32_t id, TXZ_Result result);
static void receive_handler(uint32_t id, TXZ_Result result, uart_receive_t *p_info);
static uart_io_t *safety_UART_initialize(uart_io_initial_setting_t *p_param);
static void safety_UART_finalize(uart_io_t *p_instance);

/*--------------------------------------------------*/
/** 
  * @brief  Error Hook
  * @param  -
  * @return -
  * @retval -
  * @note   for debug
  */
/*--------------------------------------------------*/
static void safety_UART_error(void)
{
#ifdef UART_IO_DEBUG
    while(1)
    {
        __nop();
    }
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  The character is shown to an output stream.
  * @param  c :Character data.
  * @return Output Character data.
  * @retval (value < 0) :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static int put_char(int c)
{
    int ret = -1;
    uart_io_private_tx_t *p_tx = &local_instance.info.tx;

    if (p_tx->state == UART_IO_SEND_IDLE)
    {
        uart_transmit_t param;
        volatile TXZ_WorkState loopBreak = TXZ_BUSY;
        uint32_t status;

        /*------------------------------*/
        /*  Transmit                    */
        /*------------------------------*/
        p_tx->buff[0] = (uint8_t)c;
        p_tx->wp      = 1;
        p_tx->rp      = 0;
        p_tx->state   = UART_IO_SEND_RUN;
        param.tx8.p_data    = &p_tx->buff[0];
        param.tx8.num       = p_tx->wp;
        if (uart_transmitIt(&local_instance.info.uart, &param) != TXZ_SUCCESS)
        {
            safety_UART_error();
        }
        /*------------------------------*/
        /*  Wait to transmit            */
        /*------------------------------*/
        loopBreak = TXZ_BUSY;
        while(loopBreak == TXZ_BUSY)
        {
            switch (p_tx->state)
            {
            case UART_IO_SEND_RUN:
                /* no process */
                break;
            case UART_IO_SEND_SUCCESS:
                if (uart_get_status(&local_instance.info.uart, &status) == TXZ_SUCCESS)
                {
                    if ((status & UART_TX_STATE_RUN) != UART_TX_STATE_RUN)
                    {
                        loopBreak = TXZ_DONE;
                    }
                }
                break;
            default:
                loopBreak = TXZ_DONE;
                break;
            }
        }
        /*------------------------------*/
        /*  Check Error                 */
        /*------------------------------*/
        if (p_tx->state == UART_IO_SEND_FAILURE)
        {
            safety_UART_error();
        }
        /* Clear status. */
        p_tx->state = UART_IO_SEND_IDLE;
        /* Set result. */
        ret = c;
    }

    return (ret);
}

/*--------------------------------------------------*/
/** 
  * @brief  The character is acquired from an input stream.
  * @param  -
  * @return Input Character data.
  * @retval (value < 0) :Failure
  * @note   -
  */
/*--------------------------------------------------*/
static int get_char(void)
{
    volatile TXZ_WorkState loopBreak = TXZ_BUSY;
    volatile int c = -1;
    uart_io_private_rx_t *p_rx = &local_instance.info.rx;

    while (loopBreak == TXZ_BUSY)
    {
        if (p_rx->state == UART_IO_RECV_RUN)
        {
            /*>>> Critical information. */
            volatile uint32_t length = p_rx->length;
            /*<<< Critical information. */

            /*------------------------------*/
            /*  Check Receive Length        */
            /*------------------------------*/
            if (length > 0)
            {
                c = (int)p_rx->buff[p_rx->rp];
                p_rx->rp++;
                if (p_rx->rp >= UART_IO_RECEIVE_MAX)
                {
                    p_rx->rp = 0;
                }
                /*>>> Critical information. */
                NVIC_DisableIRQ(local_instance.init.irq.rx);
                p_rx->length--;
                NVIC_EnableIRQ(local_instance.init.irq.rx);
                /*<<< Critical information. */
                loopBreak = TXZ_DONE;
            }
        }
        else
        {
            loopBreak = TXZ_DONE;
        }
    }

    return (c);
}

/*--------------------------------------------------*/
/** 
  * @brief  Transmit handler.
  * @param  id      :User ID.
  * @param  result  :Result.
  * @return -
  * @retval -
  * @note   Called by uart driver.
  */
/*--------------------------------------------------*/
static void transmit_handler(uint32_t id, TXZ_Result result)
{
    uart_io_t *p_instance = (uart_io_t *)id;

    if (p_instance != SAFETY_NULL)
    {
        uart_io_private_tx_t *p_tx = &p_instance->info.tx;

        if (p_tx->state != UART_IO_SEND_IDLE)
        {
            /*------------------------------*/
            /*  Result Check                */
            /*------------------------------*/
            if (result == TXZ_SUCCESS)
            {
                p_tx->state = UART_IO_SEND_SUCCESS;
            }
            else
            {
                p_tx->state = UART_IO_SEND_FAILURE;
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Receive handler.
  * @param  id      :User ID.
  * @param  result  :Result.
  * @param  p_info  :Received information Source Address.
  * @return -
  * @retval -
  * @note   Called by uart driver.
  */
/*--------------------------------------------------*/
static void receive_handler(uint32_t id, TXZ_Result result, uart_receive_t *p_info)
{
    uart_io_t *p_instance = (uart_io_t *)id;

    if ((p_instance != SAFETY_NULL) && (p_info != SAFETY_NULL))
    {
        uart_io_private_rx_t *p_rx = &p_instance->info.rx;

        if (p_rx->state == UART_IO_RECV_RUN)
        {
            if (result == TXZ_SUCCESS)
            {
                uint32_t i;
                uint32_t num    = p_info->rx8.num;
                /*>>> Critical information. */
                volatile uint32_t length = p_rx->length;
                /*<<< Critical information. */

                if (length < UART_IO_RECEIVE_MAX) 
                {
                    if ((length + num) > UART_IO_RECEIVE_MAX)
                    {
                        num = (uint32_t)(UART_IO_RECEIVE_MAX - length);
                    }
                    for (i=0; i<num; i++)
                    {
                        p_rx->buff[p_rx->wp + i] = *(p_info->rx8.p_data + i);
                        p_rx->wp++;
                        if (p_rx->wp >= UART_IO_RECEIVE_MAX)
                        {
                            p_rx->wp = 0;
                        }
                    }
                    /*>>> Critical information. */
                    p_rx->length += num;
                    /*<<< Critical information. */
                }
                else
                {
                    //p_rx->state = UART_IO_RECV_FAILURE;
                }
            }
            else
            {
                //p_rx->state = UART_IO_RECV_FAILURE;
            }
        }
    }
}

/*--------------------------------------------------*/
/** 
  * @brief  Initialize
  * @param  p_param :Initial Setting Information Source Address.
  * @return Instance Address.
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
static uart_io_t *safety_UART_initialize(uart_io_initial_setting_t *p_param)
{
    // clear id value
    local_instance.init.id = 0;      // for in case error
        
    // set baudrate
    local_instance.init.boudrate     = p_param->boudrate;

    // check UART id
    // UART base address and IRQ to use
    switch (p_param->id) {
    case SAFETY_UART0_ID:
      local_instance.init.p_reg     = TSB_UART0;
      local_instance.init.irq.tx    = INTSC0TX_IRQn;  // this will call irq_usb_uart_tx() in "bsp.c"
      local_instance.init.irq.rx    = INTSC0RX_IRQn;  // this will call irq_usb_uart_rx() in "bsp.c" 
      local_instance.init.irq.err   = INTSC0ERR_IRQn; // this will call irq_usb_uart_err() in "bsp.c"
      break;
      
    case SAFETY_UART1_ID:
      local_instance.init.p_reg     = TSB_UART1;
      local_instance.init.irq.tx    = INTSC1TX_IRQn;
      local_instance.init.irq.rx    = INTSC1RX_IRQn;
      local_instance.init.irq.err   = INTSC1ERR_IRQn;
      break;
      
    case SAFETY_UART2_ID:
      local_instance.init.p_reg     = TSB_UART2;
      local_instance.init.irq.tx    = INTSC2TX_IRQn;
      local_instance.init.irq.rx    = INTSC2RX_IRQn;
      local_instance.init.irq.err   = INTSC2ERR_IRQn;
      break;
      
    case SAFETY_UART3_ID:
      local_instance.init.p_reg     = TSB_UART3;
      local_instance.init.irq.tx    = INTSC3TX_IRQn;
      local_instance.init.irq.rx    = INTSC3RX_IRQn;
      local_instance.init.irq.err   = INTSC3ERR_IRQn;
      break;
      
    default:
#if defined ( __ICCARM__ )    /*IAR Compiler */
      DEBUGMSG(1, ("invalid UART ID (%d)\r\n", p_param->id));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
      strcpy(uart.DEBUGMSG_uart1,&DEBUGMSG_uart_temp[0][0]);
#endif
      return SAFETY_NULL;
    }

  
    /*------------------------------*/
    /*  Initialize other Variable   */
    /*------------------------------*/
    local_instance.info.tx.state      = UART_IO_SEND_IDLE;
    local_instance.info.tx.wp         = 0;
    local_instance.info.tx.rp         = 0;
    local_instance.info.rx.state      = UART_IO_RECV_RUN;     /* Auto run. */
    local_instance.info.rx.wp         = 0;
    local_instance.info.rx.rp         = 0;
    local_instance.info.rx.length     = 0;
    /*------------------------------*/
    /*  Uart Setting                */
    /*------------------------------*/
    /*--- Construct (Register Allocation) ---*/
    local_instance.info.uart.p_instance = local_instance.init.p_reg;
    /*--- Driver Initialize ---*/
    local_instance.info.uart.init.id            = (uint32_t)&local_instance;  /* User ID: Any Value (Example...UART driver instance address). */
    local_instance.info.uart.init.clock.prsel   = UART_PLESCALER_1;
    {
        cg_t paramCG;

        paramCG.p_instance = TSB_CG;
        if (uart_get_boudrate_setting(
                cg_get_phyt0(&paramCG), 
                &local_instance.info.uart.init.clock, 
                local_instance.init.boudrate, 
                &local_instance.info.uart.init.boudrate
                ) != TXZ_SUCCESS)
        {
#if defined ( __ICCARM__ )    /*IAR Compiler */
            DEBUGMSG(1, ("uart_get_boudrate_setting FAILED.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
            strcpy(uart.DEBUGMSG_uart2,&DEBUGMSG_uart_temp[1][0]);
#endif
            return SAFETY_NULL;
        }
    }
    /* Uart     */
    local_instance.info.uart.init.inttx         = UART_TX_INT_ENABLE;
    local_instance.info.uart.init.intrx         = UART_RX_INT_ENABLE;
    local_instance.info.uart.init.interr        = UART_ERR_INT_ENABLE;
    local_instance.info.uart.init.txfifo.inttx  = UART_TX_FIFO_INT_DISABLE;
    local_instance.info.uart.init.txfifo.level  = 0;
    local_instance.info.uart.init.rxfifo.intrx  = UART_RX_FIFO_INT_DISABLE;
    local_instance.info.uart.init.rxfifo.level  = 8;
    local_instance.info.uart.init.nf            = UART_NOISE_FILTER_NON;
    local_instance.info.uart.init.ctse          = UART_CTS_DISABLE;
    local_instance.info.uart.init.rtse          = UART_RTS_DISABLE;
    local_instance.info.uart.init.iv            = UART_DATA_COMPLEMENTION_DISABLE;
    local_instance.info.uart.init.dir           = UART_DATA_DIRECTION_LSB;
    local_instance.info.uart.init.sblen         = UART_STOP_BIT_1;
    local_instance.info.uart.init.even          = UART_PARITY_BIT_ODD;
    local_instance.info.uart.init.pe            = UART_PARITY_DISABLE;
    local_instance.info.uart.init.sm            = UART_DATA_LENGTH_8;
    local_instance.info.uart.transmit.handler   = transmit_handler;
    local_instance.info.uart.receive.handler    = receive_handler;
    if (uart_init(&local_instance.info.uart) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("uart_init FAILED.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(uart.DEBUGMSG_uart3,&DEBUGMSG_uart_temp[2][0]);
#endif
        return SAFETY_NULL;
    }
    /*--- Driver Start :Receive ---*/
    {
        uart_receive_t rx;

        rx.rx8.p_data = &local_instance.info.work[0];
        rx.rx8.num    = UART_RX_FIFO_MAX;
        if (uart_receiveIt(&local_instance.info.uart, &rx) != TXZ_SUCCESS)
        {
#if defined ( __ICCARM__ )    /*IAR Compiler */
            DEBUGMSG(1, ("uart_receiveId FAILED.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
            strcpy(uart.DEBUGMSG_uart4,&DEBUGMSG_uart_temp[3][0]);
#endif
            return SAFETY_NULL;
        }
    }
    
    // ALL OK
    // now set id value. this will switch IRQs
    local_instance.init.id            = p_param->id;
        
    /*------------------------------*/
    /*  Enable Interrupt            */
    /*------------------------------*/
    NVIC_EnableIRQ(local_instance.init.irq.tx);
    NVIC_EnableIRQ(local_instance.init.irq.rx);
    NVIC_EnableIRQ(local_instance.init.irq.err);
   
    return(&local_instance);
}

/*--------------------------------------------------*/
/** 
  * @brief  Finalize
  * @param  p_instance :Instance Address.
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
static void safety_UART_finalize(uart_io_t *p_instance)
{
    // clear id value
    local_instance.init.id = 0;      // for in case error
    
    /*------------------------------*/
    /*  Disable Enable              */
    /*------------------------------*/
    NVIC_DisableIRQ(local_instance.init.irq.tx);
    NVIC_DisableIRQ(local_instance.init.irq.rx);
    NVIC_DisableIRQ(local_instance.init.irq.err);
    /*------------------------------*/
    /*  Uart Finalize               */
    /*------------------------------*/
    /*--- Driver Finalize ---*/
    if (uart_deinit(&p_instance->info.uart) != TXZ_SUCCESS)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_UART_finalize FAILED.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(uart.DEBUGMSG_uart5,&DEBUGMSG_uart_temp[4][0]);
#endif
        return;
    }

    /*--- Destruct (Register release) ---*/
    p_instance->info.uart.p_instance = SAFETY_NULL;
}


/**
 *  @}
 */ /* End of group UARTTest_Private_functions */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/** 
 *  @addtogroup UARTTest_Exported_functions
 *  @{
 */

/** 
  * @brief  TX IRQ Handler.
  * @param  p_instance :Instance Address.
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void safety_UART_hook_tx(SafetyUartID id)
{
  if (id == (SafetyUartID)local_instance.init.id)
  {
    uart_transmit_irq_handler(&local_instance.info.uart);
  }
}

/*--------------------------------------------------*/
/** 
  * @brief  RX IRQ Handler.
  * @param  p_instance :Instance Address.
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void safety_UART_hook_rx(SafetyUartID id)
{
  if (id == (SafetyUartID)local_instance.init.id)
  {
    uart_receive_irq_handler(&local_instance.info.uart);
  }
}

/*--------------------------------------------------*/
/** 
  * @brief  ERR IRQ Handler.
  * @param  p_instance :Instance Address.
  * @return -
  * @retval -
  * @note   Parameter check isn't performed.
  * @attention   Singleton.
  */
/*--------------------------------------------------*/
void safety_UART_hook_err(SafetyUartID id)
{
  if (id == (SafetyUartID)local_instance.init.id)
  {
    uart_error_irq_handler(&local_instance.info.uart);
  }
}

/*--------------------------------------------------*/
/**
  * @brief     UART loopback test
  * @retval    true  :Success.
  * @note      -
  * @attention -
  * @attention -
  */
/*--------------------------------------------------*/
bool safety_UART_loopback(SafetyUartID id, uint32_t start, uint32_t length)
{
    char *p_send = (char *)start;
    char *p_recv = (char *)start;
    char *p_end = (char *)(start + length);
    int ch;
    uart_io_t   *p_uart;
    uint32_t timeout_tick;
    uint32_t start_tick, actual_tick;
    
    uart_io_initial_setting_t param = {0};

    // setup parameters
    // use id and baudrate only
    param.id         = id;
    param.boudrate   = SAFETY_TEST_BPS;
    timeout_tick     = length * 100 / (SAFETY_TEST_BPS / 100);      // watch out overflow!

    timeout_tick *= 3;       // 3 times for margin
    if (timeout_tick == 0)
    {
        timeout_tick = 1;       // avoid timeout == 0
    }
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("safety_UART_loopback: timeout set to %d msec.\r\n", timeout_tick));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(uart.DEBUGMSG_uart6,&DEBUGMSG_uart_temp[5][0]);
#endif
        
    // initialize UART
    p_uart  = safety_UART_initialize(&param);          
    if (p_uart == SAFETY_NULL)
    {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_UART_loopback: initialize FAILED.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(uart.DEBUGMSG_uart7,&DEBUGMSG_uart_temp[6][0]);
#endif
      return false;
    }
    
#ifdef USE_INTERNAL_LOOPBACK
    // enable UART loopback
    local_instance.info.uart.p_instance->CR0 |= (1 << 15);  // set LPB (bit15)     
#endif    
    
    // start loopback test
    start_tick = hal_get_tick();
    do {
      
      if (p_send < p_end)
      {
        (void)put_char((int)(*p_send++));
      }
      
      ch = get_char();
      if ((ch >= 0) && (ch == *p_recv))
      {
        p_recv++;
      }

      actual_tick = hal_get_tick() - start_tick;
      
    }
    while ((p_recv < p_end) && (actual_tick < timeout_tick));       

    
    // finalize UART
    // this will also disable loopback
    safety_UART_finalize(p_uart);

    if (p_recv < p_end)
    {
        // timeout detect
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("safety_UART_loopback: TIMEOUT after %d msec.\r\n", timeout_tick));
        DEBUGMSG(1, ("%d bytes sent, %d bytes received. %d bytes total.\r\n",
                   p_end - p_send, p_end - p_recv, length));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(uart.DEBUGMSG_uart8,&DEBUGMSG_uart_temp[7][0]);
        strcpy(uart.DEBUGMSG_uart9,&DEBUGMSG_uart_temp[8][0]);
#endif
      
      return false;
    }
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("safety_UART_loopback: SUCCESS.\r\n"));
    DEBUGMSG(1, ("timeout = %d msec, actual = %d msec.\r\n",
                 timeout_tick, actual_tick));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(uart.DEBUGMSG_uart10,&DEBUGMSG_uart_temp[9][0]);
    strcpy(uart.DEBUGMSG_uart11,&DEBUGMSG_uart_temp[10][0]);
#endif

    return true;
}

/**
 *  @}
 */ /* End of group UARTTest_Exported_functions */

/**
 *  @} 
 */ /* End of group UARTTest */

/**
 *  @} 
 */ /* End of group Safety_Library */

#ifdef __cplusplus
}
#endif /* __cplusplus */

