/** 
 *******************************************************************************
 * @file    main.c
 * @brief   CLOCK/IRQ Sample Test Application.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup CLOCK_IRQ_Sample
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "txz_sample_def.h"
#include "bsp.h"
//#include "bsp_sw.h"
#include "bsp_led.h"

// use hal.c to determine timeout
#include "txz_hal.h"
   
// safety tests minimum definition (for "bool" and "SAFETY_NULL")
#include "txz_safety_def.h"

// use INTERRUPT and CLOCK safety library
#include "safety_interrupt.h"
#include "safety_clock.h"

/*------------------------------------------------------------------------------*/
/*  Externals                                                                   */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    gpio_t      gpio_obj;            /*!< Application :GPIO. */
} instance_t;

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_main1[45];
        char DEBUGMSG_main2[20];
        char DEBUGMSG_main3[15];
        char DEBUGMSG_main4[16];
        char DEBUGMSG_main5[9];
    } debugmsg_t;
#endif
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
static instance_t instance; /*!< Instance.                    */
#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t debugmain;
    char DEBUGMSG_main_temp[5][45] = {{"INTERRUPT: Failed! result_word=0xresult_word"}, 
                                      {"INTERRUPT: SUCCESS."}, 
                                      {"CLOCK: Failed!"}, 
                                      {"CLOCK: SUCCESS."},
                                      {"Finish!."}};
#endif
/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */

// timer ID value assigned by bsp.c
// you can use any number but needs bsp.c modification sometime.
// if you use already assigned ID (i.e. BSP_TIMER_1MS and BSP_TIMER_1S) needs no bsp.c modification
// if you use new ID other than BSP_TIMER_1MS or BSP_TIMER_1S, need bsp.c modification in INTT32A?AC_IRQHandler
//#define INTERRUPT_TIMER_ID       BSP_TIMER_1MS   // = 0
#define INTERRUPT_TIMER_ID       BSP_TIMER_1S    // = 1


// t32a timer channel used for Interrupt test ( 0 - 5 )
// 0 : TSB_T32A0  - 5 : TSB_T32A5
// 0 : handler already setup by bsp.c and uses BSP_TIMER_1MS for ID
// 2 : handler already setup by bsp.c and uses BSP_TIMER_1S for ID
// 1, 3, 4, 5 : need to implement handler in bsp.c in INTT32A?AC_IRQHandler
#define INTERRUPT_TIMER_CH      2


// Clock parameter (12bits)
#define CLOCK_INPUT             (uint32_t)(0)       // 0 = fc, 1 = EHOSC

// assume monitor clock (fc) = 10MHz +- 1%, internal ref clock (IHOSC2) = 10MHz +- 10%
#define CLOCK_HIGH              (uint32_t)(0x48)      // monitor MAX / ref MIN * 64
#define CLOCK_LOW               (uint32_t)(0x39)      // monitor MIN / ref MAX * 64 
   

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }
    
    //
    // do not start SysTick IRQ in Interrupt Test
    //
       
    return (result);
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/

/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    bool result;
    uint32_t result_word;
    
    int result_code = 0;        // no error
    
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- application ---*/
    // this will turn off all LEDs
    if (application_initialize() != TXZ_SUCCESS)
    {
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/

#ifdef TEST    
    // enable SysTick Interrupt (for BUSY LOOP calibration)
    {
      // start SysTick IRQ
      const uint32_t period = 80000;  // 80MHz / 1000Hz = 1msec SysTick
     
      SysTick_Config(period);    /* systick interrupt cycle setting */

      // SysTick IRQ started
    }
#endif    
    
    // Interrupt Test
    // IRQs not enabled here
    result = safety_Interrupt(INTERRUPT_TIMER_CH, &result_word);
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("INTERRUPT: Failed! result_word=0x%04x\r\n", result_word));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main1,&DEBUGMSG_main_temp[0][0]);
#endif
      result_code = 1;
      goto test_done;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("INTERRUPT: SUCCESS.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main2,&DEBUGMSG_main_temp[1][0]);
#endif
      
    }

#ifndef TEST    
    // enable SysTick Interrupt (for safety_Clock to work)
    {
      // start SysTick IRQ
      const uint32_t period = 80000;  // 80MHz / 1000Hz = 1msec SysTick
     
      (void)SysTick_Config(period);    /* systick interrupt cycle setting */

      // SysTick IRQ started
    }
#endif
    
    // Clock Test for fc
    // Systick IRQ must be started
    result = safety_CLOCK(CLOCK_INPUT, CLOCK_HIGH, CLOCK_LOW);
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("CLOCK: Failed!\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main3,&DEBUGMSG_main_temp[2][0]);
#endif
      result_code = 2;
      goto test_done;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("CLOCK: SUCCESS.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main4,&DEBUGMSG_main_temp[3][0]);
#endif
    }
    

test_done:  

    {
      led_t *p_led0 = &instance.led[0];
      led_t *p_led1 = &instance.led[1];
      led_t *p_led2 = &instance.led[2];

      // report result
      // LED0 = SUCCESS, LED1 = INTERRUPT FAIL, LED2 = CLOCK FAIL
      switch (result_code) {
      case 0:     //  ALL SUCCESS
        led_turn_on(p_led0);
        break;
        
      case 1:     // INTERRUPT FAILED
        led_turn_on(p_led1);
        break;
        
      case 2:     // CLOCK FAILED
        led_turn_on(p_led2);
        break;
        
      default:
        // INTERNAL ERROR?
        break;
      }
    }
#if defined ( __ICCARM__ )    /*IAR Compiler */
    DEBUGMSG(1, ("Finish!.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
    strcpy(debugmain.DEBUGMSG_main5,&DEBUGMSG_main_temp[4][0]);
#endif
    
    /* dummy loop */
    while (1){
        __NOP();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    // use "hal.c" to get timeout
    hal_inc_tick();
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash reception IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_rx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */

}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    // timer IRQ ID (any number)
    //assigned by "bsp.c" or other values not used in "bsp.c"
    switch (timer)
    {
    case INTERRUPT_TIMER_ID:
        safety_interrupt_irq_handler();
        break;
    default:
        /* no processing */
        break;
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_counter(BSPCounter cnt)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer done IRQ.
  * @param  ch :DMA Channel.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(uint32_t ch)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rtc(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rmc(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RAM Parity IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_parity(void)
{
    /* Sample Soft don't use this function. */
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif
/**
 *  @} 
 */ /* End of group CLOCK_IRQ_Sample */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
