/** 
 *******************************************************************************
 * @file    main.c
 * @brief   UART Sample Test Application.
 * @version V0.1.0.2
 * $Date:: 2019-02-07 15:55:49 #$
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */
#ifdef __cplusplus
 extern "C" {
#endif
/**
 *  @addtogroup Sample_Appli
 *  @{
 */

/** 
 *  @addtogroup UART_Sample
 *  @{
 */

/** 
 *  @defgroup MAIN_PRIVATE  MAIN Private
 *  @brief    MAIN Private Group.
 *  @{
 */

/*------------------------------------------------------------------------------*/
/*  Includes                                                                    */
/*------------------------------------------------------------------------------*/
#include <string.h>
#include "txz_sample_def.h"
#include "bsp.h"
//#include "bsp_sw.h"
#include "bsp_led.h"

// use hal.c to determine timeout
#include "txz_hal.h"
   
// safety tests minimum definition (for "bool" and "SAFETY_NULL")
#include "txz_safety_def.h"

// use UART safety library
#include "safety_uart.h"

/*------------------------------------------------------------------------------*/
/*  Externals                                                                   */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Enumerated Type Definition                                                  */
/*------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------*/
/*  Structure Definition                                                        */
/*------------------------------------------------------------------------------*/
/*----------------------------------*/
/** 
 * @brief Instance Information.
*/
/*----------------------------------*/
typedef struct
{
    led_t       led[(uint32_t)BSP_LED_MAX]; /*!< Application :LED                */
    gpio_t      gpio_obj;            /*!< Application :GPIO. */

} instance_t;

#if defined ( __CC_ARM   )      /* RealView Compiler */
    typedef struct
    {
        char DEBUGMSG_main1[27];
        char DEBUGMSG_main2[28];
        char DEBUGMSG_main3[29];
        char DEBUGMSG_main4[9];
    } debugmsg_t;
#endif
/*------------------------------------------------------------------------------*/
/*  Private Member                                                              */
/*------------------------------------------------------------------------------*/
static instance_t instance; /*!< Instance.                    */

static char test_data[] = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
static SafetyUartID id;

#if defined ( __CC_ARM   )      /* RealView Compiler */
    static debugmsg_t debugmain;
    char DEBUGMSG_main_temp[4][29] = {{"SysTick Initialize failed."}, 
                                      {"UART loopback Test: FAILED."}, 
                                      {"UART loopback Test: SUCCESS."},
                                      {"Finish!."}};
#endif

#define TEST_LEN	sizeof(test_data)

/*------------------------------------------------------------------------------*/
/*  Configuration                                                               */
/*------------------------------------------------------------------------------*/
#define MAIN_NULL       ((void *)0)     /*!< NULL. */

// which UART to test?
#define SAFETY_UART0    0
#define SAFETY_UART1    1
#define SAFETY_UART2    2
#define SAFETY_UART3    3
#define SAFETY_UART_CH  SAFETY_UART2

/*------------------------------------------------------------------------------*/
/*  Private Function                                                            */
/*------------------------------------------------------------------------------*/
static TXZ_Result application_initialize(void);

/*--------------------------------------------------*/
/** 
  * @brief  Application Initialize.
  * @param  -
  * @return Result.
  * @retval TXZ_SUCCESS :Success.
  * @retval TXZ_ERROR   :Failure.
  * @note   -
  */
/*--------------------------------------------------*/
static TXZ_Result application_initialize(void)
{
   TXZ_Result result = TXZ_SUCCESS;
    /*----------------------*/
    /* LED                  */
    /*----------------------*/
    {
        uint32_t i;
        led_t *p_led = MAIN_NULL;

        for (i=0; i<(uint32_t)BSP_LED_MAX; i++)
        {
            p_led = &instance.led[i];

            p_led->init.id                  = (uint32_t)p_led;
            p_led->init.p_gpio              = bsp_get_gpio_instance_address();
            p_led->init.port.group          = bsp_get_gpio_group_led((BSPLed)(i));
            p_led->init.port.num            = bsp_get_gpio_num_led((BSPLed)(i));
            p_led->init.blink.func          = TXZ_DISABLE;
            p_led->init.blink.interval.on   = 0;
            p_led->init.blink.interval.off  = 0;
            p_led->init.state           = LED_STATE_OFF;
            led_initialize(p_led);
        }
    }
    
    /*----------------------*/
    /* SysTick               */
    /*----------------------*/    
    {
      // enable SysTick Interrupt (for TickCount)
      uint32_t period = 80000;  // 80MHz / 1000Hz = 1msec SysTick
     
      if (SysTick_Config(period))    /* systick interrupt cycle setting */
      {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("SysTick Initialize failed.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main1,&DEBUGMSG_main_temp[0][0]);
#endif
        result = TXZ_ERROR;
      }
      
      // no need this
      //NVIC_EnableIRQ(SysTick_IRQn);   /* systick interrupt enable */
    }

    /*----------------------*/
    /* UART                 */
    /*----------------------*/
    {
      // setup UART function
      gpio_t *p_gpio = bsp_get_gpio_instance_address();   // already initialized by BSP
      
#if SAFETY_UART_CH == SAFETY_UART0
      
      id = SAFETY_UART0_ID;
      // setup K0/K1 for UART0
      // this is done by "bsp.c" so no need to do it here
      //(void)gpio_func(p_gpio, GPIO_PORT_K, 0, GPIO_PK0_UT0RXD, GPIO_PIN_INPUT);
      //(void)gpio_func(p_gpio, GPIO_PORT_K, 1, GPIO_PK1_UT0TXDA, GPIO_PIN_OUTPUT);
      
#endif
#if SAFETY_UART_CH == SAFETY_UART1
      
      id = SAFETY_UART1_ID;
      // setup A0/A1 for UART1
      // PA0/PA1 default use in "bsp.c" is SPI1
      (void)gpio_func(p_gpio, GPIO_PORT_A, 0, GPIO_PA0_UT1RXD, GPIO_PIN_INPUT);
      (void)gpio_func(p_gpio, GPIO_PORT_A, 1, GPIO_PA1_UT1TXDA, GPIO_PIN_OUTPUT);
      
#endif
#if SAFETY_UART_CH == SAFETY_UART2
      
      id = SAFETY_UART2_ID;
      // setup G1/G0 for UART2 
      // PG0/PG1 default use in "bsp.c" is T32A and SPI2
      (void)gpio_func(p_gpio, GPIO_PORT_G, 1, GPIO_PG1_UT2RXD, GPIO_PIN_INPUT);
      (void)gpio_func(p_gpio, GPIO_PORT_G, 0, GPIO_PG0_UT2TXDA, GPIO_PIN_OUTPUT);
      
#endif
#if SAFETY_UART_CH == SAFETY_UART3
      
      id = SAFETY_UART3_ID;
      // setup C0/C1 for UART3
      // PG0/PG1 default use in "bsp.c" is T32A and SPI3
      (void)gpio_func(p_gpio, GPIO_PORT_C, 0, GPIO_PC0_UT3RXD, GPIO_PIN_INPUT);
      (void)gpio_func(p_gpio, GPIO_PORT_C, 1, GPIO_PC1_UT3TXDA, GPIO_PIN_OUTPUT);
      
#endif
      
    }
    
    return result;
}

/**
 *  @}
 */ /* End of group MAIN_PRIVATE */

/*------------------------------------------------------------------------------*/
/*  Public Function                                                             */
/*------------------------------------------------------------------------------*/
/*--------------------------------------------------*/
/** 
  * @brief  Main Function.
  * @param  -
  * @retval -
  * @note   -
  */
/*--------------------------------------------------*/
int main(void)
{
    bool result;
    
    int result_code = 0;        // no error
      
    /*----------------------*/
    /* BSP initialize       */
    /*----------------------*/
    bsp_initialize();   // Initialize PK0 and PK1 for UART0 use
    /*----------------------*/
    /* Initialize           */
    /*----------------------*/
    /*--- application ---*/
    // this will turn off all LEDs
    if (application_initialize() != TXZ_SUCCESS)
    {
    }
    /*----------------------*/
    /* Main Processing      */
    /*----------------------*/

    // UART loopback test
    result = safety_UART_loopback(id, (uint32_t)test_data, TEST_LEN);
    if (!result) {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("UART loopback Test: FAILED.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main2,&DEBUGMSG_main_temp[1][0]);
#endif
      result_code = 1;
    }
    else {
#if defined ( __ICCARM__ )    /*IAR Compiler */
        DEBUGMSG(1, ("UART loopback Test: SUCCESS.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
        strcpy(debugmain.DEBUGMSG_main3,&DEBUGMSG_main_temp[2][0]);
#endif
    }
    
    

    {
      led_t *p_led0 = &instance.led[0];
      led_t *p_led1 = &instance.led[1];

      // report result
      // LED0 = SUCCESS, LED1 = UART loopback Test FAIL
      switch (result_code) {
      case 0:     //  ALL SUCCESS
        led_turn_on(p_led0);
        break;
        
      case 1:     // UART loopback test FAILED
        led_turn_on(p_led1);
        break;
        
      default:
        // INTERNAL ERROR?
        break;
      }
    }
#if defined ( __ICCARM__ )    /*IAR Compiler */
     DEBUGMSG(1, ("Finish!.\r\n"));
#elif defined ( __CC_ARM   )      /* RealView Compiler */
     strcpy(debugmain.DEBUGMSG_main4,&DEBUGMSG_main_temp[3][0]);
#endif
    
    /* dummy loop */
    while (1){
        __NOP();
    }
}
/*--------------------------------------------------*/
/** 
  * @brief  Systick IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_systick(void)
{
    // use "hal.c" to get timeout
    hal_inc_tick();
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash reception IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_rx(BSPSFlash sflash)
{
  // IRQ handler for safety_UART_loopback
  // UART1/2/3 use this IRQ for RX
#if SAFETY_UART_CH==SAFETY_UART1
  if (sflash == BSP_SFLASH_1)
  {
    safety_UART_hook_rx(SAFETY_UART1_ID);
  }
#endif
#if SAFETY_UART_CH==SAFETY_UART2
  if (sflash == BSP_SFLASH_2)
  {
    safety_UART_hook_rx(SAFETY_UART2_ID);
  }
#endif
#if SAFETY_UART_CH==SAFETY_UART3
  if (sflash == BSP_SFLASH_3)
  {
    safety_UART_hook_rx(SAFETY_UART3_ID);
  }
#endif
  
  // add normal UART/SPI IRQ handler here.
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash transmit IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_tx(BSPSFlash sflash)
{
  // IRQ handler for safety_UART_loopback
  // UART1/2/3 use this IRQ for Tx
#if SAFETY_UART_CH==SAFETY_UART1
  if (sflash == BSP_SFLASH_1)
  {
    safety_UART_hook_tx(SAFETY_UART1_ID);
  }
#endif
#if SAFETY_UART_CH==SAFETY_UART2
  if (sflash == BSP_SFLASH_2)
  {
    safety_UART_hook_tx(SAFETY_UART2_ID);
  }
#endif
#if SAFETY_UART_CH==SAFETY_UART3
  if (sflash == BSP_SFLASH_3)
  {
    safety_UART_hook_tx(SAFETY_UART3_ID);
  }
#endif
  
  // add normal UART/SPI IRQ handler here.
}

/*--------------------------------------------------*/
/** 
  * @brief  SPI flash error IRQ.
  * @param  sflash :SPI Flash
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_sflash_err(BSPSFlash sflash)
{
  // IRQ handler for safety_UART_loopback
  // UART1/2/3 use this IRQ for Error
#if SAFETY_UART_CH==SAFETY_UART1
  if (sflash == BSP_SFLASH_1)
  {
    safety_UART_hook_err(SAFETY_UART1_ID);
  }
#endif
#if SAFETY_UART_CH==SAFETY_UART2
  if (sflash == BSP_SFLASH_2)
  {
    safety_UART_hook_err(SAFETY_UART2_ID);
  }
#endif
#if SAFETY_UART_CH==SAFETY_UART3
  if (sflash == BSP_SFLASH_3)
  {
    safety_UART_hook_err(SAFETY_UART3_ID);
  }
#endif
  
  // add normal UART/SPI IRQ handler here.
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART reception IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_rx(BSPUsbUart uart)
{
  // UART0 use this IRQ for RX
#if SAFETY_UART_CH==SAFETY_UART0
    safety_UART_hook_rx(SAFETY_UART0_ID);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART transmit IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_tx(BSPUsbUart uart)
{
  // UART0 use this IRQ for TX
#if SAFETY_UART_CH==SAFETY_UART0
    safety_UART_hook_tx(SAFETY_UART0_ID);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  USB UART error IRQ.
  * @param  uart :USB UART.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_usb_uart_err(BSPUsbUart uart)
{
  // UART0 use this IRQ for Error
#if SAFETY_UART_CH==SAFETY_UART0
    safety_UART_hook_err(SAFETY_UART0_ID);
#endif
}

/*--------------------------------------------------*/
/** 
  * @brief  1ms Timer.
  * @param  timer :1ms Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_timer(BSPTimer timer)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  Timer.
  * @param  timer :Timer.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_counter(BSPCounter cnt)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer done IRQ.
  * @param  ch :DMA Channel.
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_done(uint32_t ch)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  DMA transfer error IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_dmac_err(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RTC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rtc(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RMC IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_rmc(void)
{
    /* Sample Soft don't use this function. */
}
/*--------------------------------------------------*/
/** 
  * @brief  I2C IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_i2c(void)
{
    /* Sample Soft don't use this function. */
}

/*--------------------------------------------------*/
/** 
  * @brief  RAM Parity IRQ.
  * @param  -
  * @return -
  * @retval -
  * @note   This function is called BSP.
  */
/*--------------------------------------------------*/
void irq_parity(void)
{
    /* Sample Soft don't use this function. */
}

#ifdef DEBUG
/*--------------------------------------------------*/
/** 
  * @brief  Failure Assert.
  * @note   for debug
  */
/*--------------------------------------------------*/
void assert_failed(char *file, int32_t line)
{
    while (1) {
        __nop();
    }
}
#endif
/**
 *  @} 
 */ /* End of group UART_Sample */

/**
 *  @} 
 */ /* End of group Sample_Appli */

#ifdef __cplusplus
}
#endif /* __cplusplus */
